import java.io.IOException;
import java.util.Collection;

import javax.jdo.Extent;
import javax.jdo.PersistenceManager;
import javax.jdo.Query;
import javax.jdo.Transaction;

import org.eclipse.example.library.Book;
//import org.eclipse.example.library.BookCategory;
import org.eclipse.example.library.Library;
import org.eclipse.example.library.LibraryFactory;
import org.eclipse.example.library.Writer;
import org.eclipse.example.library.impl.LibraryImpl;

import org.eclipse.example.library.db.PM;
import org.eclipse.example.library.db.bookCategoryEnum;

public class Add {

	/** 
	 * @param args
	 */
	public static void main(String[] args) throws IOException {
		
 		// Now create a persistence manager and a transaction
       	PersistenceManager pm = PM.createPersistenceManager();
       	Transaction tx = pm.currentTransaction(); 
 
       	// at this point the database contains one library, one book and one writer
       	// Reopen the transaction and query for the library objects 
       	tx.begin();
    	
       	// NOTE: here you see one drawback of the current JPOX implementation:
       	// for querying you require the concrete implementation instead of the
       	// interface!
       	// retrieve all LibraryImpl classes and subclasses.
	    Extent e = pm.getExtent(LibraryImpl.class, true);
	    Query q = pm.newQuery(e);

	    // there is only one library
	    Collection c = (Collection)q.execute();
	    Library lib = (Library)c.iterator().next();
	    
	    // read the writer and book
	    Writer writer = (Writer)lib.getWriters().get(0);
	    System.out.println(writer.getName());
	    Book book = (Book)lib.getBooks().get(0);
	    System.out.println(book.getTitle());
	    
	    // show that the container is set
	    System.out.println(book.eContainer() == lib);
	    System.out.println(writer.getBooks().get(0) == book);
	    
	    // Now add a new writer and book
	    Writer george = LibraryFactory.eINSTANCE.createWriter();
	    george.setName("G. Orwell");
	    
	    // create a new book and set the writer and library
        Book georgesBook = LibraryFactory.eINSTANCE.createBook();
        georgesBook.setPages(250);
        georgesBook.setTitle("1984");
    	
        georgesBook.setCategory(bookCategoryEnum.create(pm, "Science Fiction"));
        georgesBook.setAuthor(george);

        lib.getBooks().add(georgesBook);
        lib.getWriters().add(george);
        
        // and close of
	    tx.commit();
	    pm.close();
	}
}
