import java.io.IOException;

import javax.jdo.PersistenceManager;
import javax.jdo.Transaction;

import org.eclipse.example.library.Book;
//import org.eclipse.example.library.BookCategory;
import org.eclipse.example.library.Library;
import org.eclipse.example.library.LibraryFactory;
import org.eclipse.example.library.Writer;
import org.eclipse.example.library.db.PM;
import org.eclipse.example.library.db.bookCategoryEnum;

public class Init {

	/** 
	 * @param args
	 */
	public static void main(String[] args) throws IOException
{
		
		// Now create a persistence manager and a transaction
       	PersistenceManager pm = PM.createPersistenceManager();
       	Transaction tx = pm.currentTransaction(); 
        try
        {
	    	
	    	// start a transaction, create a library and make it persistent
	    	tx.begin();
	    	Library lib = LibraryFactory.eINSTANCE.createLibrary();
	    	lib.setName("My Library");
	    	pm.makePersistent(lib);
	    	
	    	// create a writer
	    	Writer writer = LibraryFactory.eINSTANCE.createWriter();
	    	writer.setName("JRR Tolkien");
	       
	    	// and one of his books
	    	Book book = LibraryFactory.eINSTANCE.createBook();
	    	book.setAuthor(writer);
	    	book.setPages(305);
	    	book.setTitle("The Hobbit");
	    	
	    //	BookCategory bookCategorySF = LibraryFactory.eINSTANCE.createBookCategory();
	    //	bookCategorySF.setName("Science Fiction");	
	    	book.setCategory(bookCategoryEnum.create(pm, "Science Fiction"));
	
	    	// add the writer/book to the library. The writer and book are automatically
	    	// made persistent because they are added to the library which is already
	    	// made persistent
	    	lib.getWriters().add(writer);
	    	lib.getBooks().add(book);
	        
	    	// at commit the objects will be present in the database
	    	tx.commit();
        }
        finally
        {
            if (tx.isActive())
            {
                tx.rollback();
            }
            	pm.close(); //empty the cache
        }
	}
	
}
