package org.eclipse.example.library;

import java.util.Collection;

import javax.jdo.Extent;
import javax.jdo.PersistenceManager;
import javax.jdo.PersistenceManagerFactory;
import javax.jdo.Query;
import javax.jdo.Transaction;

import org.eclipse.example.library.impl.LibraryImpl;
import org.elver.store.jpox.emf.JPOXHelper;

public class Tutorial1 {

	/** 
	 * @param args
	 */
	public static void main(String[] args) {
		
    	// At this position you can add your own specific types for jpox
    	// This should be done before creating the persistence manager factory, e.g.
    	//TypeManager.getTypeManager().addType(MyNiceType.class.getName(), MyNiceTypeMapping.class, MyNiceTypeWrapper.class);
    	
    	// Now create and register a PersistenceManagerFactory
    	String pmfName = "MyPMF"; // the name of the persistence manager factory
    	
    	String dbDriver = ""; // the database driver
    	String dbName = ""; // the name of the database, this database should exist (can be empty)
    	String dbUrl = ""; //for mysql the databasename is attached to the dburl
    	String dbUser = ""; // the database user
    	String dbPwd = ""; // the database password of this user.

    	String db = "derby";
    //	if (db.compareTo("pgsql") == true) {
        	if ( true ) {
    	 dbDriver = "org.apache.derby.jdbc.ClientDriver"; // the database driver
        	 dbName = "mylibrary"; // the name of the database, this database should exist (can be empty)
        	 dbUrl = "jdbc:derby://127.0.0.1:1527/" + dbName; //for mysql the databasename is attached to the dburl
        	 dbUser = "user"; // the database user
        	 dbPwd = "pass"; // the database password of this user.
    	} else {
    		dbDriver = "org.postgresqlDriver"; // the database driver
    		dbName = "mylibrary"; // the name of the database, this database should exist (can be empty)
    		dbUrl = "jdbc:postgresql://127.0.0.1/" + dbName; //for mysql the databasename is attached to the dburl
    		dbUser = "pgsql"; // the database user
    		dbPwd = "pgsql"; // the database password of this user.
    	}
    	PersistenceManagerFactory pmf = JPOXHelper.INSTANCE.createRegisterPersistenceManagerFactory(
    			pmfName, LibraryPackage.eINSTANCE, dbDriver, dbUrl, dbUser, dbPwd, dbName);
    	
    	// Now create a persistence manager and a transaction
    	PersistenceManager pm = pmf.getPersistenceManager();
    	Transaction tx = pm.currentTransaction(); 
    	
    	// start a transaction, create a library and make it persistent
    	tx.begin();
    	Library lib = LibraryFactory.eINSTANCE.createLibrary();
    	lib.setName("My Library");
    	pm.makePersistent(lib);
    	
    	// create a writer
    	Writer writer = LibraryFactory.eINSTANCE.createWriter();
    	writer.setName("JRR Tolkien");
       
    	// and one of his books
    	Book book = LibraryFactory.eINSTANCE.createBook();
    	book.setAuthor(writer);
    	book.setPages(305);
    	book.setTitle("The Hobbit");
    	
    	BookCategory bookCategorySF = LibraryFactory.eINSTANCE.createBookCategory();
    	bookCategorySF.setName("Science Fiction");	
    	book.setCategory(bookCategorySF);

    	// add the writer/book to the library. The writer and book are automatically
    	// made persistent because they are added to the library which is already
    	// made persistent
    	lib.getWriters().add(writer);
    	lib.getBooks().add(book);
        
    	// at commit the objects will be present in the database
    	tx.commit();
    	pm.close(); //empty the cache

    	pm = pmf.getPersistenceManager();
    	tx = pm.currentTransaction(); 
    	
    	// at this point the database contains one library, one book and one writer
    	// Reopen the transaction and query for the library objects 
    	tx.begin();
    	
    	// NOTE: here you see one drawback of the current JPOX implementation:
    	// for querying you require the concrete implementation instead of the
    	// interface!
    	// retrieve all LibraryImpl classes and subclasses.
	    Extent e = pm.getExtent(LibraryImpl.class, true);
	    Query q = pm.newQuery(e);

	    // there is only one library
	    Collection c = (Collection)q.execute();
	    lib = (Library)c.iterator().next();
	    
	    // read the writer and book
	    writer = (Writer)lib.getWriters().get(0);
	    System.out.println(writer.getName());
	    book = (Book)lib.getBooks().get(0);
	    System.out.println(book.getTitle());
	    
	    // show that the container is set
	    System.out.println(book.eContainer() == lib);
	    System.out.println(writer.getBooks().get(0) == book);
	    
	    // Now add a new writer and book
	    Writer george = LibraryFactory.eINSTANCE.createWriter();
	    george.setName("G. Orwell");
	    
	    // create a new book and set the writer and library
        Book georgesBook = LibraryFactory.eINSTANCE.createBook();
        georgesBook.setPages(250);
        georgesBook.setTitle("1984");
        georgesBook.setCategory(bookCategorySF);
        georgesBook.setAuthor(george);

        lib.getBooks().add(georgesBook);
        lib.getWriters().add(george);
        
        // and close of
	    tx.commit();
	    pm.close();

    	pm = pmf.getPersistenceManager();
    	tx = pm.currentTransaction();
    	
	    tx.begin();

	    // retrieve all books
	    Query qry = pm.newQuery("SELECT FROM org.eclipse.example.library.impl.BookImpl");
	    Collection coll = (Collection)qry.execute();
	    System.out.println(((Book)coll.iterator().next()).getTitle()); // show a title
	    System.out.println(((Book)coll.iterator().next()).getTitle()); // show a title

	    // retrieve a book which has a writer with the name of G. Orwell 
	    qry = pm.newQuery("SELECT FROM org.eclipse.example.library.impl.BookImpl WHERE " +
	    		" title==\"1984\" && author == writ && writ.name == \"G. Orwell\" " + 
	    		"VARIABLES org.eclipse.example.library.impl.WriterImpl writ");
	    coll = (Collection)qry.execute();
	    System.out.println(coll.size()); // should be 1
	    Book bk = (Book)coll.iterator().next();
	    System.out.println(bk.getTitle()); //should be 1984
    	System.out.println(bk.getAuthor().getName()); //should be G. Orwell
	    
    	// read the library with a name ending on Library
	    qry = pm.newQuery("SELECT FROM org.eclipse.example.library.impl.LibraryImpl WHERE name.endsWith(\"Library\")");
	    coll = (Collection)qry.execute();
	    lib = (Library)coll.iterator().next();
	    tx.commit(); 
	}
}
