import java.io.IOException;

import javax.jdo.PersistenceManager;
import javax.jdo.Transaction;

import com.ibm.example.projecttree.Leaf;
import com.ibm.example.projecttree.Dir;
import com.ibm.example.projecttree.ProjectTree;
import com.ibm.example.projecttree.ProjecttreeFactory;
import com.ibm.example.projecttree.NativeFile;
import com.ibm.example.projecttree.PipedFile;
import com.ibm.example.projecttree.ZipFile;

import com.ibm.example.projecttree.db.PM;

public class Init {

	/** 
	 * @param args
	 */
	public static void main(String[] args) throws IOException
{
		
    	// At this position you can add your own specific types for jpox
    	// This should be done before creating the persistence manager factory, e.g.
    	//TypeManager.getTypeManager().addType(MyNiceType.class.getName(), MyNiceTypeMapping.class, MyNiceTypeWrapper.class);
    	
    	// Now create a persistence manager and a transaction
       	PersistenceManager pm = PM.createPersistenceManager();
       	Transaction tx = pm.currentTransaction(); 
        try
        {
	    	
	    	// start a transaction, create a projecttree and make it persistent
	    	tx.begin();
	    	ProjectTree projecttree = ProjecttreeFactory.eINSTANCE.createProjectTree();
	    	projecttree.setName("project");
	    	pm.makePersistent(projecttree);
	    	
	    	// create a dist dir
	    	Dir dist_dir = ProjecttreeFactory.eINSTANCE.createDir();
	    	dist_dir.setName("/usr/pkg/stage3.2M3-elver/project/dist");
	       
	    	// create a src dir
	    	Dir src_dir = ProjecttreeFactory.eINSTANCE.createDir();
	    	src_dir.setName("/usr/pkg/stage3.2M3-elver/project/src");
	       
	    	// and it's content
	    	ZipFile zipfile = ProjecttreeFactory.eINSTANCE.createZipFile();
	    	zipfile.setName(projecttree.getName()+".zip");
	    	
	    	// add the dir/zipfile to the projecttree. The dir and zipfile are automatically
	    	// made persistent because they are added to the projecttree which is already
	    	// made persistent
	    	projecttree.getDirs().add(dist_dir);
	    	projecttree.getDirs().add(src_dir);
	    	projecttree.getLeafs().add(zipfile);
	        
	    	// at commit the objects will be present in the database
	    	tx.commit();
        }
        finally
        {
            if (tx.isActive())
            {
                tx.rollback();
            }
            	pm.close(); //empty the cache
        }
	}
	
}
